/*
Copyright © 2022 NAME HERE <EMAIL ADDRESS>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

	http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"context"
	"os/signal"
	"syscall"

	"github.com/minio/minio-go/v7"
	"github.com/minio/minio-go/v7/pkg/credentials"
	"github.com/nats-io/nats.go"
	"github.com/spf13/cobra"
	"libre.sh/nextcloud-version-retention/internal"
)

var natsUrl, s3Endpoint, s3Key, s3Secret string
var dryRun bool

var mc *minio.Client
var nc *nats.Conn

var rootCmd = &cobra.Command{
	Use:   "nextcloud-version-retention",
	Short: "Process version retention on s3:ObjectCreated events",
	PersistentPreRunE: func(cmd *cobra.Command, args []string) (err error) {
		nc, err = nats.Connect(natsUrl)
		if err != nil {
			return
		}

		mc, err = minio.New(s3Endpoint, &minio.Options{
			Creds:  credentials.NewStaticV4(s3Key, s3Secret, ""),
			Secure: true,
		})
		return
	},
	PersistentPostRun: func(cmd *cobra.Command, args []string) {
		nc.Drain()
		nc.Close()
	},
	Run: func(cmd *cobra.Command, args []string) {
		ctx, cancel := signal.NotifyContext(context.Background(), syscall.SIGTERM, syscall.SIGINT)
		defer cancel()
		internal.Process(ctx, mc, nc, dryRun)
	},
}

func Execute() {
	cobra.CheckErr(rootCmd.Execute())
}

func init() {
	rootCmd.PersistentFlags().StringVar(&natsUrl, "nats-url", nats.DefaultURL, "Nats url.")
	rootCmd.PersistentFlags().StringVar(&s3Endpoint, "s3-endpoint", "", "S3 endpoint without scheme.")
	rootCmd.PersistentFlags().StringVar(&s3Key, "s3-access-key", "", "S3 access key.")
	rootCmd.PersistentFlags().StringVar(&s3Secret, "s3-access-secret", "", "S3 access secret.")
	rootCmd.Flags().BoolVar(&dryRun, "dry-run", false, "Don't remove object version.")
}
